// $Id: Transformer.js,v 1.1.2.13 2005/08/18 18:59:35 vnagarjuna Exp $ -->

//Copyright 2005 Nagarjuna Venna <vnagarjuna@yahoo.com>

/* ****** BEGIN LICENSE BLOCK ***** 
 *  This file is part of Padma.
 *
 *  Padma is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Padma is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Padma; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * ***** END LICENSE BLOCK ***** */

//Transformer

//Input/output methods
Transformer.method_RTS      = 0;
Transformer.method_Unicode  = 1;
Transformer.method_DynFonts = 2;
Transformer.method_Unknown  = 3;

//Dynamic fonts supported
Transformer.dynFont_Eenadu       = 0;
Transformer.dynFont_ShreeTel0900 = 1;
Transformer.dynFont_Hemalatha    = 2;
Transformer.dynFont_ShreeTel0902 = 3;
Transformer.dynFont_Tikkana      = 4;
Transformer.dynFont_Unknown      = 5;   //this should always be the max, this is used as a sentinel in whitelist.js

Transformer.dynFont_name = new Array();
Transformer.dynFont_name[Transformer.dynFont_Eenadu]       = "Eenadu";
Transformer.dynFont_name[Transformer.dynFont_ShreeTel0900] = "Shree-Tel-0900";
Transformer.dynFont_name[Transformer.dynFont_Hemalatha]    = "TLW-TTHemalatha";
Transformer.dynFont_name[Transformer.dynFont_ShreeTel0902] = "Shree-Tel-0902";
Transformer.dynFont_name[Transformer.dynFont_Tikkana]      = "Tikkana";

Transformer.padmaNameToIndexMap = new Array();
Transformer.padmaNameToIndexMap[Transformer.dynFont_name[Transformer.dynFont_Eenadu].toUpperCase()]       = Transformer.dynFont_Eenadu;
Transformer.padmaNameToIndexMap[Transformer.dynFont_name[Transformer.dynFont_ShreeTel0900].toUpperCase()] = Transformer.dynFont_ShreeTel0900;
Transformer.padmaNameToIndexMap[Transformer.dynFont_name[Transformer.dynFont_Hemalatha].toUpperCase()]    = Transformer.dynFont_Hemalatha;
Transformer.padmaNameToIndexMap[Transformer.dynFont_name[Transformer.dynFont_ShreeTel0902].toUpperCase()] = Transformer.dynFont_ShreeTel0902;
Transformer.padmaNameToIndexMap[Transformer.dynFont_name[Transformer.dynFont_Tikkana].toUpperCase()]      = Transformer.dynFont_Tikkana;

Transformer.padmaIndexToNameMap = new Array();
Transformer.padmaIndexToNameMap[Transformer.dynFont_Eenadu]       = Transformer.dynFont_name[Transformer.dynFont_Eenadu];
Transformer.padmaIndexToNameMap[Transformer.dynFont_ShreeTel0900] = Transformer.dynFont_name[Transformer.dynFont_ShreeTel0900];
Transformer.padmaIndexToNameMap[Transformer.dynFont_Hemalatha]    = Transformer.dynFont_name[Transformer.dynFont_Hemalatha];
Transformer.padmaIndexToNameMap[Transformer.dynFont_ShreeTel0902] = Transformer.dynFont_name[Transformer.dynFont_ShreeTel0902];
Transformer.padmaIndexToNameMap[Transformer.dynFont_Tikkana]      = Transformer.dynFont_name[Transformer.dynFont_Tikkana];

function Transformer(input, output, rtsWritingStyle, rtsSunnaStyle)
{
    this.input = input;
    this.output = output;
    this.dynFontIndex = Transformer.dynFont_Unknown;
    this.rtsWriter =  ((this.output == Transformer.method_RTS) ? RTS.getRTSWriter(rtsWritingStyle, rtsSunnaStyle) : null);
}

Transformer.createTransformer = function (input, output, rtsWritingStyle, rtsSunnaStyle)
{
    if (!Transformer.isValidOutputMethod(output) || !Transformer.isValidInputMethod(input))
        return null;
    if (input == Transformer.method_RTS)
        return new RTSTransformer(input, output);
    if (output == Transformer.method_RTS)
        return new Transformer(input, output, rtsWritingStyle, rtsSunnaStyle);
    return new Transformer(input, output);
}

Transformer.prototype.getInputMethod = function ()
{
    return this.input;
}

Transformer.prototype.getOutputMethod = function ()
{
    return this.output;
}

Transformer.isValidInputMethod = function (method)
{
    return method == Transformer.method_RTS || method == Transformer.method_DynFonts || method == Transformer.method_Unicode 
           || method == Transformer.method_Unknown;
}

Transformer.isValidOutputMethod = function (method)
{
    return method == Transformer.method_RTS || method == Transformer.method_Unicode;
}

Transformer.prototype.convert = function (text)
{
    //Convert to internal format from inputMethod
    var output = "";
    var parser;
    if (this.input == Transformer.method_DynFonts || 
        (this.input == Transformer.method_Unknown && this.dynFontIndex != Transformer.dynFont_Unknown)) {
        switch (this.dynFontIndex) {
        case Transformer.dynFont_Eenadu:
            parser = new DynamicFontParser(text, Eenadu);
            break;
            
        case Transformer.dynFont_ShreeTel0900:
        case Transformer.dynFont_ShreeTel0902:
            parser = new DynamicFontParser(text, Shree_Tel_0900);
            break;

        case Transformer.dynFont_Hemalatha:
            parser = new DynamicFontParser(text, Hemalatha);
            break;

        case Transformer.dynFont_Tikkana:
            parser = new DynamicFontParser(text, Tikkana);
            break;

        default:
            parser = null;
            break;
        }
        this.dynFontIndex = Transformer.dynFont_Unknown;
    }
    else if (this.input == Transformer.method_Unicode || this.input == Transformer.method_Unknown) {
        parser = new UnicodeParser(text);
    }

    while (parser.more())
        output += this.toOutputFormat(parser.next());

    if (this.rtsWriter != null)
        output += this.rtsWriter.cleanup();

    return output;
}

Transformer.prototype.toOutputFormat = function (syllable)
{
    if (this.output == Transformer.method_Unicode)
        return Unicode.transformFromPadma(syllable);
    if (this.output == Transformer.method_RTS)
        return this.rtsWriter.transformFromPadma(syllable);
    //Assert?
    return syllable;
}

Transformer.isSupportedDynamicFont = function (face)
{
    return Transformer.padmaNameToIndexMap[face.toUpperCase()] != null;
}

Transformer.getDynFontIndex = function (face)
{
    var index = Transformer.padmaNameToIndexMap[face.toUpperCase()];
    return index != null ? index : Transformer.dynFont_Unknown;
}

Transformer.prototype.setDynamicFont = function (face)
{
    if (this.input != Transformer.method_DynFonts && this.input != Transformer.method_Unknown)
        return false;
    var index = Transformer.padmaNameToIndexMap[face.toUpperCase()];
    if (index == null)
        return false;
    this.dynFontIndex = index;
    return true;
}

Transformer.prototype.setDynamicFontByIndex = function (index)
{
    if (this.input != Transformer.method_DynFonts && this.input != Transformer.method_Unknown)
        return false;
    if (Transformer.dynFont_name[index] == null)
        return false;
    this.dynFontIndex = index;
    return true;
}
